@@ -574,12 +574,12 @@ RELEASE_MAPPING = {
         }
     },
     'master': {
-        'api': '1.83',
-        'rpc': '1.55',
+        'api': '1.86',
+        'rpc': '1.56',
         'objects': {
             'Allocation': ['1.1'],
             'BIOSSetting': ['1.1'],
-            'Node': ['1.38', '1.37'],
+            'Node': ['1.39', '1.38', '1.37'],
             'NodeHistory': ['1.0'],
             'NodeInventory': ['1.0'],
             'Conductor': ['1.3'],
@@ -592,6 +592,7 @@ RELEASE_MAPPING = {
             'TraitList': ['1.0'],
             'VolumeConnector': ['1.0'],
             'VolumeTarget': ['1.0'],
+            'FirmwareComponent': ['1.0'],
         }
     },
 }
