@@ -105,6 +105,12 @@ class BareDriver(object):
     A reference to an instance of :class:DeployInterface.
     """
 
+    firmware = None
+    """`Standard` attribute for inspection related features.
+
+    A reference to an instance of :class:FirmwareInterface.
+    """
+
     inspect = None
     """`Standard` attribute for inspection related features.
 
@@ -161,7 +167,8 @@ class BareDriver(object):
     @property
     def optional_interfaces(self):
         """Interfaces that can be no-op."""
-        return ['bios', 'console', 'inspect', 'raid', 'rescue', 'storage']
+        return ['bios', 'console', 'firmware', 'inspect', 'raid', 'rescue',
+                'storage']
 
     @property
     def all_interfaces(self):
@@ -226,8 +233,9 @@ class BaseInterface(object, metaclass=abc.ABCMeta):
         """
 
     def __new__(cls, *args, **kwargs):
-        # Get the list of clean steps and deploy steps, when the interface is
-        # initialized by the conductor. We use __new__ instead of __init___
+        # Get the list of clean steps, deploy steps and verify steps when the
+        # interface is initialized by the conductor.
+        # We use __new__ instead of __init___
         # to avoid breaking backwards compatibility with all the drivers.
         # We want to return all steps, regardless of priority.
 
@@ -238,6 +246,7 @@ class BaseInterface(object, metaclass=abc.ABCMeta):
             instance = super_new(cls, *args, **kwargs)
         instance.clean_steps = []
         instance.deploy_steps = []
+        instance.verify_steps = []
         for n, method in inspect.getmembers(instance, inspect.ismethod):
             if getattr(method, '_is_clean_step', False):
                 # Create a CleanStep to represent this method
@@ -245,7 +254,9 @@ class BaseInterface(object, metaclass=abc.ABCMeta):
                         'priority': method._clean_step_priority,
                         'abortable': method._clean_step_abortable,
                         'argsinfo': method._clean_step_argsinfo,
-                        'interface': instance.interface_type}
+                        'interface': instance.interface_type,
+                        'requires_ramdisk':
+                            method._clean_step_requires_ramdisk}
                 instance.clean_steps.append(step)
             if getattr(method, '_is_deploy_step', False):
                 # Create a DeployStep to represent this method
@@ -254,6 +265,13 @@ class BaseInterface(object, metaclass=abc.ABCMeta):
                         'argsinfo': method._deploy_step_argsinfo,
                         'interface': instance.interface_type}
                 instance.deploy_steps.append(step)
+            if getattr(method, '_is_verify_step', False):
+                # Create a VerifyStep to represent this method
+                step = {'step': method.__name__,
+                        'priority': method._verify_step_priority,
+                        'interface': instance.interface_type}
+                instance.verify_steps.append(step)
+
         if instance.clean_steps:
             LOG.debug('Found clean steps %(steps)s for interface '
                       '%(interface)s',
@@ -264,6 +282,12 @@ class BaseInterface(object, metaclass=abc.ABCMeta):
                       '%(interface)s',
                       {'steps': instance.deploy_steps,
                        'interface': instance.interface_type})
+        if instance.verify_steps:
+            LOG.debug('Found verify steps %(steps)s for interface '
+                      '%(interface)s',
+                      {'steps': instance.deploy_steps,
+                       'interface': instance.interface_type})
+
         return instance
 
     def _execute_step(self, task, step):
@@ -358,6 +382,35 @@ class BaseInterface(object, metaclass=abc.ABCMeta):
         """
         return self._execute_step(task, step)
 
+    def get_verify_steps(self, task):
+        """Get a list of (enabled and disabled) verify steps for the interface.
+
+        This function will return all verify steps (both enabled and disabled)
+        for the interface, in an unordered list.
+
+        :param task: A TaskManager object, useful for interfaces overriding
+            this function
+        :raises NodeVerifyFailure: if there is a problem getting the steps
+            from the driver. For example, when a node (using an agent driver)
+            has just been enrolled and the agent isn't alive yet to be queried
+            for the available verify steps.
+        :returns: A list of deploy step dictionaries
+        """
+        return self.verify_steps
+
+    def execute_verify_step(self, task, step):
+        """Execute the verify step on task.node.
+
+        A verify step must take a single positional argument: a TaskManager
+        object. It does not take keyword variable arguments.
+
+        :param task: A TaskManager object
+        :param step: The deploy step dictionary representing the step to
+            execute
+        :returns: None if this method has completed synchronously
+        """
+        return self._execute_step(task, step)
+
 
 class DeployInterface(BaseInterface):
     """Interface for deploy-related actions."""
@@ -476,13 +529,16 @@ class DeployInterface(BaseInterface):
         pass
 
     def heartbeat(self, task, callback_url, agent_version,
-                  agent_verify_ca=None):
+                  agent_verify_ca=None, agent_status=None,
+                  agent_status_message=None):
         """Record a heartbeat for the node.
 
         :param task: A TaskManager instance containing the node to act on.
         :param callback_url: a URL to use to call to the ramdisk.
         :param agent_version: The version of the agent that is heartbeating
         :param agent_verify_ca: TLS certificate for the agent.
+        :param agent_status: Status of the heartbeating agent
+        :param agent_status_message: Message describing the agent status
         :return: None
         """
         LOG.warning('Got heartbeat message from node %(node)s, but '
@@ -1159,12 +1215,23 @@ class ManagementInterface(BaseInterface):
         raise exception.UnsupportedDriverExtension(
             driver=task.node.driver, extension='detect_vendor')
 
+    def get_mac_addresses(self, task):
+        """Get MAC address information for the node.
+
+        :param task: A TaskManager instance containing the node to act on.
+        :raises: UnsupportedDriverExtension
+        :returns: A list of MAC addresses for the node
+
+        """
+        raise exception.UnsupportedDriverExtension(
+            driver=task.node.driver, extension='get_mac_addresses')
+
 
 class InspectInterface(BaseInterface):
     """Interface for inspection-related actions."""
     interface_type = 'inspect'
 
-    ESSENTIAL_PROPERTIES = {'memory_mb', 'local_gb', 'cpus', 'cpu_arch'}
+    ESSENTIAL_PROPERTIES = {'memory_mb', 'local_gb', 'cpu_arch'}
     """The properties required by scheduler/deploy."""
 
     @abc.abstractmethod
@@ -1197,6 +1264,20 @@ class InspectInterface(BaseInterface):
         raise exception.UnsupportedDriverExtension(
             driver=task.node.driver, extension='abort')
 
+    def continue_inspection(self, task, inventory, plugin_data=None):
+        """Continue in-band hardware inspection.
+
+        Should not be implemented for purely out-of-band implementations.
+
+        :param task: a task from TaskManager.
+        :param inventory: hardware inventory from the node.
+        :param plugin_data: optional plugin-specific data.
+        :raises: UnsupportedDriverExtension, if the method is not implemented
+                 by specific inspect interface.
+        """
+        raise exception.UnsupportedDriverExtension(
+            driver=task.node.driver, extension='abort')
+
 
 def cache_bios_settings(func):
     """A decorator to cache bios settings after running the function.
@@ -1602,7 +1683,7 @@ class NetworkInterface(BaseInterface):
         """
 
     def need_power_on(self, task):
-        """Check if ironic node must be powered on before applying network changes
+        """Check if node must be powered on before applying network changes
 
         :param task: A TaskManager instance.
         :returns: Boolean.
@@ -1662,6 +1743,55 @@ class StorageInterface(BaseInterface, metaclass=abc.ABCMeta):
         """
 
 
+def cache_firmware_components(func):
+    """A decorator to cache firmware components after running the function.
+
+    :param func: Function or method to wrap.
+    """
+    @functools.wraps(func)
+    def wrapped(self, task, *args, **kwargs):
+        result = func(self, task, *args, **kwargs)
+        self.cache_firmware_components(task)
+        return result
+    return wrapped
+
+
+class FirmwareInterface(BaseInterface):
+    """Base class for firmware interface"""
+
+    interface_type = 'firmware'
+
+    @abc.abstractmethod
+    def update(self, task, settings):
+        """Update the Firmware on the given using the settings for components.
+
+        :param task: a TaskManager instance.
+        :param settings: a list of dictionaries, each dictionary contains the
+            component name and the url that will be used to update the
+            firmware.
+        :raises: UnsupportedDriverExtension, if the node's driver doesn't
+            support update via the interface.
+        :raises: InvalidParameterValue, if validation of the settings fails.
+        :raises: MissingParamterValue, if some required parameters are
+            missing.
+        :returns: states.CLEANWAIT if Firmware update with the settings is in
+            progress asynchronously of None if it is complete.
+        """
+
+    @abc.abstractmethod
+    def cache_firmware_components(self, task):
+        """Store or update Firmware Components on the given node.
+
+        This method stores Firmware Components to the firmware_information
+        table during 'cleaning' operation. It will also update the timestamp
+        of each Firmware Component.
+
+        :param task: a TaskManager instance.
+        :raises: UnsupportedDriverExtension, if the node's driver doesn't
+            support getting Firmware Components from bare metal.
+        """
+
+
 def _validate_argsinfo(argsinfo):
     """Validate args info.
 
@@ -1716,7 +1846,8 @@ def _validate_argsinfo(argsinfo):
                 {'arg': arg})
 
 
-def clean_step(priority, abortable=False, argsinfo=None):
+def clean_step(priority, abortable=False, argsinfo=None,
+               requires_ramdisk=True):
     """Decorator for cleaning steps.
 
     Cleaning steps may be used in manual or automated cleaning.
@@ -1770,6 +1901,8 @@ def clean_step(priority, abortable=False, argsinfo=None):
             'required': Boolean. Optional; default is False. True if this
                         argument is required.  If so, it must be specified in
                         the clean request; false if it is optional.
+    :param requires_ramdisk: Whether this step requires the ramdisk
+        to be running. Should be set to False for purely out-of-band steps.
     :raises InvalidParameterValue: if any of the arguments are invalid
     """
     def decorator(func):
@@ -1790,6 +1923,7 @@ def clean_step(priority, abortable=False, argsinfo=None):
 
         _validate_argsinfo(argsinfo)
         func._clean_step_argsinfo = argsinfo
+        func._clean_step_requires_ramdisk = requires_ramdisk
         return func
     return decorator
 
@@ -1848,3 +1982,44 @@ def deploy_step(priority, argsinfo=None):
         func._deploy_step_argsinfo = argsinfo
         return func
     return decorator
+
+
+def verify_step(priority):
+    """Decorator for verify steps.
+
+    Only steps with priorities greater than 0 are used. These steps are
+    ordered by priority from highest value to lowest value.
+    For steps with the same priority, they are ordered by driver
+    interface priority (see conductor.steps.VERIFY_INTERFACE_PRIORITY).
+    execute_verify_step() will be called on each step.
+
+    Decorated verify steps must take as the only positional argument, a
+    TaskManager object.
+
+    Verify steps are synchronous and should return `None` when finished,
+    and the conductor will continue on to the next step. While the verify step
+    is executing, the node will be in `states.VERIFYING` provision state.
+
+    Examples::
+
+        class MyInterface(base.BaseInterface):
+            @base.verify_step(priority=100)
+            def example_verifying(self, task):
+                # do some verifying
+
+    :param priority: an integer (>=0) priority; used for determining the order
+        in which the step is run in the verification process.
+
+    :raises InvalidParameterValue: if any of the arguments are invalid
+    """
+    def decorator(func):
+        func._is_verify_step = True
+        if isinstance(priority, int) and priority >= 0:
+            func._verify_step_priority = priority
+        else:
+            raise exception.InvalidParameterValue(
+                _('"priority" must be an integer value >= 0, instead of "%s"')
+                % priority)
+
+        return func
+    return decorator
