@@ -14,7 +14,7 @@
 #    under the License.
 
 from ironic.drivers import generic
-from ironic.drivers.modules import inspector
+from ironic.drivers.modules import agent
 from ironic.drivers.modules import ipxe
 from ironic.drivers.modules import noop
 from ironic.drivers.modules import noop_mgmt
@@ -24,6 +24,7 @@ from ironic.drivers.modules.redfish import boot as redfish_boot
 from ironic.drivers.modules.redfish import inspect as redfish_inspect
 from ironic.drivers.modules.redfish import management as redfish_mgmt
 from ironic.drivers.modules.redfish import power as redfish_power
+from ironic.drivers.modules.redfish import raid as redfish_raid
 from ironic.drivers.modules.redfish import vendor as redfish_vendor
 
 
@@ -48,8 +49,8 @@ class RedfishHardware(generic.GenericHardware):
     @property
     def supported_inspect_interfaces(self):
         """List of supported power interfaces."""
-        return [redfish_inspect.RedfishInspect, inspector.Inspector,
-                noop.NoInspect]
+        return ([redfish_inspect.RedfishInspect]
+                + super().supported_inspect_interfaces)
 
     @property
     def supported_boot_interfaces(self):
@@ -63,3 +64,8 @@ class RedfishHardware(generic.GenericHardware):
     def supported_vendor_interfaces(self):
         """List of supported vendor interfaces."""
         return [redfish_vendor.RedfishVendorPassthru, noop.NoVendor]
+
+    @property
+    def supported_raid_interfaces(self):
+        """List of supported raid interfaces."""
+        return [redfish_raid.RedfishRAID, noop.NoRAID, agent.AgentRAID]
