@@ -77,14 +77,12 @@ class TestACLBase(base.BaseApiTest):
     def _fake_process_request(self, request, auth_token_request):
         pass
 
-    def _test_request(self, path, params=None, headers=None, method='get',  # noqa: C901, E501
+    def _test_request(self, path, params=None, headers=None, method='get',
                       body=None, assert_status=None,
                       assert_dict_contains=None,
                       assert_list_length=None,
                       deprecated=None,
-                      self_manage_nodes=True,
-                      enable_service_project=False,
-                      service_project='service'):
+                      self_manage_nodes=True):
         path = path.format(**self.format_data)
         self.mock_auth.side_effect = self._fake_process_request
 
@@ -94,13 +92,6 @@ class TestACLBase(base.BaseApiTest):
                 'project_admin_can_manage_own_nodes',
                 False,
                 'api')
-        if enable_service_project:
-            cfg.CONF.set_override('rbac_service_role_elevated_access', True)
-        if service_project != 'service':
-            # Enable us to sort of gracefully test a name variation
-            # with existing ddt test modeling.
-            cfg.CONF.set_override('rbac_service_project_name',
-                                  service_project)
 
         # always request the latest api version
         version = api_versions.max_version_string()
@@ -294,6 +285,10 @@ class TestRBACModelBeforeScopesBase(TestACLBase):
             value=fake_setting)
         db_utils.create_test_node_trait(
             node_id=fake_db_node['id'])
+        # Create a Fake Firmware Component BMC
+        db_utils.create_test_firmware_component(
+            node_id=fake_db_node['id'],
+        )
         fake_history = db_utils.create_test_history(node_id=fake_db_node.id)
         fake_inventory = db_utils.create_test_inventory(
             node_id=fake_db_node.id)
@@ -401,6 +396,17 @@ class TestRBACProjectScoped(TestACLBase):
             owner=owner_project_id,
             last_error='meow',
             reservation='lolcats')
+        # invisible child node, rbac + project query enforcement
+        # prevents it from being visible.
+        db_utils.create_test_node(
+            uuid='2b3b8adb-add7-4fd0-8e82-dcb714d848e7',
+            parent_node=owned_node.uuid)
+        # Child node which will appear in child node endpoint
+        # queries.
+        db_utils.create_test_node(
+            uuid='3c3b8adb-edd7-3ed0-8e82-aab714d8411a',
+            parent_node=owned_node.uuid,
+            owner=owner_project_id)
         owned_node_port = db_utils.create_test_port(
             uuid='ebe30f19-358d-41e1-8d28-fd7357a0164c',
             node_id=owned_node['id'],
